==> videosdk-0.0.7-py3-none-any.whl: videosdk/meeting.py <==
import base64
import gc
import json
import logging
from typing import Callable, Optional

import requests
from .pubsub import PubSub
from .utils import RecordingConfig, TranscriptionConfig
from .custom_audio_track import CustomAudioTrack
from .custom_video_track import CustomVideoTrack
from ._events import Events, VideoSDKEvents
from .event_handler import BaseEvent
from .lib.pymediasoup.producer import Producer
from .participant import Participant
from .room_client import RoomClient


class Meeting:
    """
    A class to represent a meeting.

    Attributes:
        meeting_id (str): The ID of the meeting.
        local_participant (Participant): The local participant of the meeting.
        room_client (RoomClient): The room client managing the meeting.
    """

    def __init__(
        self, meeting_id: str, local_participant: Participant, room_client: RoomClient
    ):
        """
        Initialize the meeting with the given meeting ID, local participant, and room client.

        Args:
            meeting_id (str): The ID of the meeting.
            local_participant (Participant): The local participant of the meeting.
            room_client (RoomClient): The room client managing the meeting.
        """
        self.__id = meeting_id
        self.__local_participant = local_participant
        self.__room_client = room_client
        self.__listeners: list[BaseEvent] = []
        self.__participants: dict[str, Participant] = {}
        self.__active_speaker_id: str = None
        self.__internal_event_listeners: dict[str, Callable] = {}
        self.__pubsub = PubSub(room_client=room_client)
        self.__base_url: str = None
        self.__recording_state: str = None
        self.__transcription_state: str = None
        logging.debug(f"Meeting initialized with ID: {meeting_id}")
        self.__listen()

    def __del__(self):
        """Log when the meeting instance is deleted."""
        logging.debug("Meeting :: instance deleted %s", self.__id)

    def join(self):
        """Join the meeting."""
        logging.debug(f"Starting meeting {self.__id}")
        self.__room_client.join()

    async def async_join(self):
        """Asynchronously Join the meeting."""
        logging.debug(f"Starting meeting {self.__id}")
        await self.__room_client.async_join()

    def leave(self):
        """Leave the meeting."""
        logging.debug(f"Leaving meeting {self.__id}")
        self.__room_client.leave()

    def end(self):
        """End the meeting."""
        logging.debug(f"Ending/Closing running meeting {self.__id}")
        self.__room_client.close()

    def enable_mic(self, track: Optional[CustomAudioTrack] = None):
        """
        Enable the microphone.

        Args:
            track (Optional[CustomAudioTrack]): Custom audio track to use.
        """
        logging.debug(f"Unmute mic {self.__id}")
        self.__room_client.enable_mic(track)

    def disable_mic(self):
        """Disable the microphone."""
        logging.debug(f"Mute mic {self.__id}")
        self.__room_client.disable_mic()

    def enable_webcam(self, track: Optional[CustomVideoTrack] = None):
        """
        Enable the webcam.

        Args:
            track (Optional[CustomVideoTrack]): Custom video track to use.
        """
        logging.debug(f"Webcam on {self.__id}")
        self.__room_client.enable_webcam(track)

    def disable_webcam(self):
        """Disable the webcam."""
        logging.debug(f"Webcam off {self.__id}")
        self.__room_client.disable_webcam()

    def start_recording(self, recording_config: Optional[RecordingConfig] = None):
        """
        Start recording the meeting.

        Args:
            recording_config (Optional[RecordingConfig]): Configuration for the recording.
        """
        logging.debug(f"recording starting {self.__id}")
        self.__room_client.start_recording(recording_config)

    def stop_recording(self):
        """Stop recording the meeting."""
        logging.debug(f"recording stopping {self.__id}")
        self.__room_client.stop_recording()

    def start_transcription(self, transcription_config: Optional[TranscriptionConfig] = None):
        """
        Start transcription of the meeting.

        Args:
            transcription_config (Optional[TranscriptionConfig]): Configuration for the transcription.
        """
        logging.debug(f"transcription starting {self.__id}")
        self.__room_client.start_transcription(transcription_config)

    def stop_transcription(self):
        """Stop transcription of the meeting."""
        logging.debug(f"transcription stopping {self.__id}")
        self.__room_client.stop_transcription()

    def release(self):
        """Release the meeting resources."""
        logging.debug(f"releasing meeting")
        self.__room_client.release()

    def add_custom_video_track(self, track):
        """Add a custom video track to the meeting."""
        self.__room_client.add_custom_video_track(track)

    def add_custom_audio_track(self, track):
        """Add a custom audio track to the meeting."""
        self.__room_client.add_custom_audio_track(track)

    def fetch_base64(self, url, token):
        """fetch base64 file from videosdk temporary storage"""
        headers = {
            'Authorization': token,
        }

        response = requests.get(url, headers=headers)
        response.raise_for_status()

        binary_data = response.content
        base64_data = base64.b64encode(binary_data).decode('utf-8')
        return base64_data

    def upload_base64(self, base64_data, token, file_name):
        """upload base64 data to videosdk temporary storage"""
        if not base64_data or not token or not file_name:
            raise ValueError(
                "Please provide base64_data, token, and file_name")

        base_url = self.__base_url
        room_id = self.id
        url = f"https://{base_url}/base64-upload?roomId={room_id}"
        request_body = {
            "fileName": file_name,
            "base64Data": json.dumps(base64_data),
        }

        headers = {
            "Content-Type": "application/json",
            "Authorization": token,
        }

        response = requests.post(
            url, json=request_body,
            headers=headers
        )
        response.raise_for_status()
        response_data = response.json()

        return response_data['url']

    def __reject_mic_request(self):
        """Reject the microphone request."""
        pass

    def __reject_webcam_request(self):
        """Reject the webcam request."""
        pass

    def add_event_listener(self, event_listener: BaseEvent):
        """
        Add an event listener to the meeting.

        Args:
            event_listener (BaseEvent): The event listener to add.
        """
        self.__listeners.append(event_listener)

    def remove_event_listener(self, event_listener: BaseEvent):
        """
        Remove an event listener from the meeting.

        Args:
            event_listener (BaseEvent): The event listener to remove.
        """
        self.__listeners.remove(event_listener)

    def remove_all_listeners(self):
        """Remove all event listeners from the meeting."""
        self.__listeners.clear()

    def _cleanup(self):
        """Cleanup internal event listeners."""
        for k, v in self.__internal_event_listeners.items():
            self.__room_client.remove_listener(k, v)

    def __emit(self, event_name, data=None):
        """
        Emit an event to all listeners.

        Args:
            event_name (str): The name of the event.
            data (Optional): Additional data to pass with the event.
        """
        for _listener in self.__listeners:
            _listener.handle_event(event_name, data)

    def __listen(self):
        """Set up internal event listeners for the meeting."""
        @self.__room_client.on(Events.ERROR)
        def on_error(data):
            self.__emit(VideoSDKEvents.EV_ERROR.value, data)

        # meeting
        @self.__room_client.on(Events.MEETING_JOINED)
        def on_meeting_joined(data):
            self.__base_url = data["base_url"]
            self.__emit(VideoSDKEvents.EV_MEETING_JOINED.value, data)

        @self.__room_client.on(Events.MEETING_LEFT)
        def on_meeting_left(data):
            self.__emit(VideoSDKEvents.EV_MEETING_LEFT.value, data)

        @self.__room_client.on(Events.MEETING_STATE_CHANGED)
        def on_meeting_state_change(data):
            self.__emit(VideoSDKEvents.EV_MEETING_STATE_CHANGE.value, data)

        @self.__room_client.on(Events.MIC_REQUESTED)
        def on_mic_requested(data):
            self.__emit(
                VideoSDKEvents.EV_MIC_REQUESTED,
                data={
                    "participantId": data["peerId"],
                    "accept": self.unmute,
                    "reject": self.__reject_mic_request,
                },
            )

        @self.__room_client.on(Events.WEBCAM_REQUESTED)
        def on_webcam_requested(data):
            self.__emit(
                VideoSDKEvents.EV_WEBCAM_REQUESTED,
                data={
                    "participantId": data["peerId"],
                    "accept": self.enable_webcam,
                    "reject": self.__reject_webcam_request,
                },
            )

        # active speaker
        @self.__room_client.on(Events.SET_ROOM_ACTIVE_SPEAKER)
        def on_speaker_changed(data):
            self.__active_speaker_id = data["peerId"] if "peerId" in data else None
            self.__emit(VideoSDKEvents.EV_SPEAKER_CHANGED.value,
                        self.__active_speaker_id)

        # participant
        @self.__room_client.on(Events.ADD_PEER)
        def on_participant_joined(data):
            id = data["id"]
            display_name = data["displayName"]
            device = data["device"]
            mode = data["mode"]
            meta_data = data["metaData"] if "metaData" in data else None

            participant = Participant(
                id=id,
                display_name=display_name,
                local=False,
                mode=mode,
                meta_data=meta_data,
                room_client=self.__room_client,
            )
            self.__participants[id] = participant
            self.__emit(
                VideoSDKEvents.EV_PARTICIPANT_JOINED.value, participant)

        @self.__room_client.on(Events.REMOVE_PEER)
        def on_participant_left(data):
            id = data["peerId"]
            if id in self.__participants:
                participant = self.__participants.pop(id)
                self.__emit(
                    VideoSDKEvents.EV_PARTICIPANT_LEFT.value, participant)

                # remove all event listeners
                participant.remove_all_listeners()
                participant._cleanup()
                # delete participant object
                del participant

                gc.collect()

        @self.__room_client.on(Events.ADD_PRODUCER)
        def on_stream_enabled(data: Producer):
            if self.__local_participant:
                self.__local_participant._add_stream(data)

        @self.__room_client.on(Events.REMOVE_PRODUCER)
        def on_stream_disabled(data: Producer):
            if self.__local_participant:
                self.__local_participant._remove_stream(data)

        # recording
        @self.__room_client.on(Events.RECORDING_STATE_CHANGED)
        def on_recording_state_changed(data):
            self.__recording_state = data["status"]
            self.__emit(VideoSDKEvents.EV_RECORDING_STATE_CHANGED.value, data)

        @self.__room_client.on(Events.RECORDING_STARTED)
        def on_recording_started(data):
            self.__emit(VideoSDKEvents.EV_RECORDING_STARTED.value, data)

        @self.__room_client.on(Events.RECORDING_STOPPED)
        def on_recording_stopped(data):
            self.__emit(VideoSDKEvents.EV_RECORDING_STOPPED.value, data)

        # realtime transcription

        @self.__room_client.on(Events.TRANSCRIPTION_STATE_CHANGED)
        def on_transcription_state_changed(data):
            self.__transcription_state = data["status"]
            self.__emit(
                VideoSDKEvents.EV_TRANSCRIPTION_STATE_CHANGED.value, data)

        @self.__room_client.on(Events.TRANSCRIPTION_TEXT)
        def on_transcription_text(data):
            self.__emit(VideoSDKEvents.EV_TRANSCRIPTION_TEXT.value, data)

    @property
    def listeners(self):
        return self.__listeners

    @property
    def id(self):
        return self.__id

    @property
    def local_participant(self):
        return self.__local_participant

    @property
    def participants(self):
        return self.__participants

    @property
    def active_speaker_id(self):
        return self.__active_speaker_id

    @property
    def pubsub(self):
        return self.__pubsub

    @property
    def recording_state(self):
        return self.__recording_state

    @property
    def transcription_state(self):
        return self.__transcription_state

==> videosdk-0.0.11-py3-none-any.whl: videosdk/meeting.py <==
import base64
import gc
import json
import logging
from typing import Callable, Optional

import requests
from .pubsub import PubSub
from .utils import RecordingConfig, TranscriptionConfig
from .custom_audio_track import CustomAudioTrack
from .custom_video_track import CustomVideoTrack
from ._events import Events, VideoSDKEvents
from .event_handler import BaseEvent
from .lib.pymediasoup.producer import Producer
from .participant import Participant
from .room_client import RoomClient

logger = logging.getLogger(__name__)


class Meeting:
    """
    A class to represent a meeting.

    Attributes:
        meeting_id (str): The ID of the meeting.
        local_participant (Participant): The local participant of the meeting.
        room_client (RoomClient): The room client managing the meeting.
    """

    def __init__(
        self, meeting_id: str, local_participant: Participant, room_client: RoomClient
    ):
        """
        Initialize the meeting with the given meeting ID, local participant, and room client.

        Args:
            meeting_id (str): The ID of the meeting.
            local_participant (Participant): The local participant of the meeting.
            room_client (RoomClient): The room client managing the meeting.
        """
        self.__id = meeting_id
        self.__local_participant = local_participant
        self.__room_client = room_client
        self.__listeners: list[BaseEvent] = []
        self.__participants: dict[str, Participant] = {}
        self.__active_speaker_id: str = None
        self.__internal_event_listeners: dict[str, Callable] = {}
        self.__pubsub = PubSub(room_client=room_client)
        self.__base_url: str = None
        self.__recording_state: str = None
        self.__transcription_state: str = None
        logger.debug(f"Meeting initialized with ID: {meeting_id}")
        self.__listen()

    def __del__(self):
        """Log when the meeting instance is deleted."""
        logger.debug("Meeting :: instance deleted %s", self.__id)

    def join(self):
        """Join the meeting."""
        logger.debug(f"Starting meeting {self.__id}")
        self.__room_client.join()

    async def async_join(self):
        """Asynchronously Join the meeting."""
        logger.debug(f"Starting meeting {self.__id}")
        await self.__room_client.async_join()

    def leave(self):
        """Leave the meeting."""
        logger.debug(f"Leaving meeting {self.__id}")
        self.__room_client.leave()

    def end(self):
        """End the meeting."""
        logger.debug(f"Ending/Closing running meeting {self.__id}")
        self.__room_client.close()

    def enable_mic(self, track: Optional[CustomAudioTrack] = None):
        """
        Enable the microphone.

        Args:
            track (Optional[CustomAudioTrack]): Custom audio track to use.
        """
        logger.debug(f"Unmute mic {self.__id}")
        self.__room_client.enable_mic(track)

    def disable_mic(self):
        """Disable the microphone."""
        logger.debug(f"Mute mic {self.__id}")
        self.__room_client.disable_mic()

    def enable_webcam(self, track: Optional[CustomVideoTrack] = None):
        """
        Enable the webcam.

        Args:
            track (Optional[CustomVideoTrack]): Custom video track to use.
        """
        logger.debug(f"Webcam on {self.__id}")
        self.__room_client.enable_webcam(track)

    def disable_webcam(self):
        """Disable the webcam."""
        logger.debug(f"Webcam off {self.__id}")
        self.__room_client.disable_webcam()

    def start_recording(self, recording_config: Optional[RecordingConfig] = None):
        """
        Start recording the meeting.

        Args:
            recording_config (Optional[RecordingConfig]): Configuration for the recording.
        """
        logger.debug(f"recording starting {self.__id}")
        self.__room_client.start_recording(recording_config)

    def stop_recording(self):
        """Stop recording the meeting."""
        logger.debug(f"recording stopping {self.__id}")
        self.__room_client.stop_recording()

    def start_transcription(
        self, transcription_config: Optional[TranscriptionConfig] = None
    ):
        """
        Start transcription of the meeting.

        Args:
            transcription_config (Optional[TranscriptionConfig]): Configuration for the transcription.
        """
        logger.debug(f"transcription starting {self.__id}")
        self.__room_client.start_transcription(transcription_config)

    def stop_transcription(self):
        """Stop transcription of the meeting."""
        logger.debug(f"transcription stopping {self.__id}")
        self.__room_client.stop_transcription()

    def release(self):
        """Release the meeting resources."""
        logger.debug(f"releasing meeting")
        self.__room_client.release()

    def add_custom_video_track(self, track):
        """Add a custom video track to the meeting."""
        self.__room_client.add_custom_video_track(track)

    def add_custom_audio_track(self, track):
        """Add a custom audio track to the meeting."""
        self.__room_client.add_custom_audio_track(track)

    def fetch_base64(self, url, token):
        """fetch base64 file from videosdk temporary storage"""
        headers = {
            "Authorization": token,
        }

        response = requests.get(url, headers=headers)
        response.raise_for_status()

        binary_data = response.content
        base64_data = base64.b64encode(binary_data).decode("utf-8")
        return base64_data

    def upload_base64(self, base64_data, token, file_name):
        """upload base64 data to videosdk temporary storage"""
        if not base64_data or not token or not file_name:
            raise ValueError("Please provide base64_data, token, and file_name")

        base_url = self.__base_url
        room_id = self.id
        url = f"https://{base_url}/base64-upload?roomId={room_id}"
        request_body = {
            "fileName": file_name,
            "base64Data": json.dumps(base64_data),
        }

        headers = {
            "Content-Type": "application/json",
            "Authorization": token,
        }

        response = requests.post(url, json=request_body, headers=headers)
        response.raise_for_status()
        response_data = response.json()

        return response_data["url"]

    def __reject_mic_request(self):
        """Reject the microphone request."""
        pass

    def __reject_webcam_request(self):
        """Reject the webcam request."""
        pass

    def add_event_listener(self, event_listener: BaseEvent):
        """
        Add an event listener to the meeting.

        Args:
            event_listener (BaseEvent): The event listener to add.
        """
        self.__listeners.append(event_listener)

    def remove_event_listener(self, event_listener: BaseEvent):
        """
        Remove an event listener from the meeting.

        Args:
            event_listener (BaseEvent): The event listener to remove.
        """
        self.__listeners.remove(event_listener)

    def remove_all_listeners(self):
        """Remove all event listeners from the meeting."""
        self.__listeners.clear()

    def _cleanup(self):
        """Cleanup internal event listeners."""
        for k, v in self.__internal_event_listeners.items():
            self.__room_client.remove_listener(k, v)

    def __emit(self, event_name, data=None):
        """
        Emit an event to all listeners.

        Args:
            event_name (str): The name of the event.
            data (Optional): Additional data to pass with the event.
        """
        for _listener in self.__listeners:
            _listener.handle_event(event_name, data)

    def __listen(self):
        """Set up internal event listeners for the meeting."""

        @self.__room_client.on(Events.ERROR)
        def on_error(data):
            self.__emit(VideoSDKEvents.EV_ERROR.value, data)

        # meeting
        @self.__room_client.on(Events.MEETING_JOINED)
        def on_meeting_joined(data):
            self.__base_url = data["base_url"]
            self.__emit(VideoSDKEvents.EV_MEETING_JOINED.value, data)

        @self.__room_client.on(Events.MEETING_LEFT)
        def on_meeting_left(data):
            self.__emit(VideoSDKEvents.EV_MEETING_LEFT.value, data)

        @self.__room_client.on(Events.MEETING_STATE_CHANGED)
        def on_meeting_state_change(data):
            self.__emit(VideoSDKEvents.EV_MEETING_STATE_CHANGE.value, data)

        @self.__room_client.on(Events.MIC_REQUESTED)
        def on_mic_requested(data):
            self.__emit(
                VideoSDKEvents.EV_MIC_REQUESTED,
                data={
                    "participantId": data["peerId"],
                    "accept": self.enable_mic,
                    "reject": self.__reject_mic_request,
                },
            )

        @self.__room_client.on(Events.WEBCAM_REQUESTED)
        def on_webcam_requested(data):
            self.__emit(
                VideoSDKEvents.EV_WEBCAM_REQUESTED,
                data={
                    "participantId": data["peerId"],
                    "accept": self.enable_webcam,
                    "reject": self.__reject_webcam_request,
                },
            )

        # active speaker
        @self.__room_client.on(Events.SET_ROOM_ACTIVE_SPEAKER)
        def on_speaker_changed(data):
            self.__active_speaker_id = data["peerId"] if "peerId" in data else None
            self.__emit(
                VideoSDKEvents.EV_SPEAKER_CHANGED.value, self.__active_speaker_id
            )

        # participant
        @self.__room_client.on(Events.ADD_PEER)
        def on_participant_joined(data):
            id = data["id"]
            display_name = data["displayName"]
            device = data["device"]
            mode = data["mode"]
            meta_data = data["metaData"] if "metaData" in data else None

            participant = Participant(
                id=id,
                display_name=display_name,
                local=False,
                mode=mode,
                meta_data=meta_data,
                room_client=self.__room_client,
                device=device,
            )
            self.__participants[id] = participant
            self.__emit(VideoSDKEvents.EV_PARTICIPANT_JOINED.value, participant)

        @self.__room_client.on(Events.REMOVE_PEER)
        def on_participant_left(data):
            id = data["peerId"]
            if id in self.__participants:
                participant = self.__participants.pop(id)
                self.__emit(VideoSDKEvents.EV_PARTICIPANT_LEFT.value, participant)

                # remove all event listeners
                participant.remove_all_listeners()
                participant._cleanup()
                # delete participant object
                del participant

                gc.collect()

        @self.__room_client.on(Events.ADD_PRODUCER)
        def on_stream_enabled(data: Producer):
            if self.__local_participant:
                self.__local_participant._add_stream(data)

        @self.__room_client.on(Events.REMOVE_PRODUCER)
        def on_stream_disabled(data: Producer):
            if self.__local_participant:
                self.__local_participant._remove_stream(data)

        # recording
        @self.__room_client.on(Events.RECORDING_STATE_CHANGED)
        def on_recording_state_changed(data):
            self.__recording_state = data["status"]
            self.__emit(VideoSDKEvents.EV_RECORDING_STATE_CHANGED.value, data)

        @self.__room_client.on(Events.RECORDING_STARTED)
        def on_recording_started(data):
            self.__emit(VideoSDKEvents.EV_RECORDING_STARTED.value, data)

        @self.__room_client.on(Events.RECORDING_STOPPED)
        def on_recording_stopped(data):
            self.__emit(VideoSDKEvents.EV_RECORDING_STOPPED.value, data)

        # realtime transcription

        @self.__room_client.on(Events.TRANSCRIPTION_STATE_CHANGED)
        def on_transcription_state_changed(data):
            self.__transcription_state = data["status"]
            self.__emit(VideoSDKEvents.EV_TRANSCRIPTION_STATE_CHANGED.value, data)

        @self.__room_client.on(Events.TRANSCRIPTION_TEXT)
        def on_transcription_text(data):
            self.__emit(VideoSDKEvents.EV_TRANSCRIPTION_TEXT.value, data)

    @property
    def listeners(self):
        return self.__listeners

    @property
    def id(self):
        return self.__id

    @property
    def local_participant(self):
        return self.__local_participant

    @property
    def participants(self):
        return self.__participants

    @property
    def active_speaker_id(self):
        return self.__active_speaker_id

    @property
    def pubsub(self):
        return self.__pubsub

    @property
    def recording_state(self):
        return self.__recording_state

    @property
    def transcription_state(self):
        return self.__transcription_state
